/* may only be used in accordance with the terms of the GNU General Public License */

#include <stdio.h>
#include <stdlib.h>

#include <gtk/gtk.h>

#include "gharmonic.h"

#define G_DEFHARM     5

#define G_RANGE     200.0

#define G_PAGE       10.0
#define G_STEP        1.0

#define G_DEFREQ    440.0
#define G_MAXFREQ 12000.0
#define G_MINFREQ    40.0

void g_initharmonic(struct g_harmonic *g, int n)
{
  g->g_harmonics = n + 1;

#ifdef TRACE
  fprintf(stderr, "g_initharmonic(): harmonics=<%d>\n", n);
#endif

  if (NULL == (g->g_adjs = (GtkObject **) malloc((n + 1) * sizeof(GtkObject *)))) {
    g_error("Can not allocate memory for <%d> object pointers\n", n + 1);
    exit(1);
  }
}

void g_emit(struct g_harmonic *g)
{
  int i;
  printf("%.0f", ((GtkAdjustment *) (g->g_adjs[0]))->value);
  for (i = 1; i < (g->g_harmonics); i++) {
    /* yuk ! I have not found a wave of flipping a slider (top<->bottom) */
#ifdef TRACE
    fprintf(stderr, "g_emit(): i=<%d>, g->adjs[%d]=<%p>\n", i, i, g->g_adjs[i]);
#endif
    printf(":%.0f", G_RANGE - (((GtkAdjustment *) (g->g_adjs[i]))->value));
  }
  printf("\n");
  fflush(stdout);
}

void g_destroy(GtkWidget * widget, gpointer data)
{
  gtk_main_quit();
}

void g_silence(GtkWidget * widget, gpointer data)
{
  printf("silence\n");
  fflush(stdout);
}

void g_notice(GtkWidget * widget, gpointer data)
{
  struct g_harmonic *g;

  g = (struct g_harmonic *) data;

  g_emit(g);
}

/* the shame - a global */
static GtkWidget *g_dumpwindow = NULL;

void g_dumpok(GtkWidget * widget, gpointer data)
{
  printf("d %s\n", gtk_file_selection_get_filename(GTK_FILE_SELECTION(g_dumpwindow)));

  gtk_widget_destroy(g_dumpwindow);
  g_dumpwindow = NULL;
}

void g_destroydump(GtkWidget * widget, gpointer data)
{
  gtk_widget_destroy(g_dumpwindow);
  g_dumpwindow = NULL;
}

void g_dump(GtkWidget * widget, gpointer data)
{

  if (g_dumpwindow == NULL) {

    g_dumpwindow = gtk_file_selection_new("Save as Text");
    gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(g_dumpwindow));
    gtk_window_position(GTK_WINDOW(g_dumpwindow), GTK_WIN_POS_MOUSE);

    gtk_signal_connect(GTK_OBJECT(g_dumpwindow), "destroy", GTK_SIGNAL_FUNC(g_destroydump), NULL);
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(g_dumpwindow)->cancel_button), "clicked", GTK_SIGNAL_FUNC(g_destroydump), NULL);
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(g_dumpwindow)->ok_button), "clicked", GTK_SIGNAL_FUNC(g_dumpok), NULL);

    gtk_widget_show(g_dumpwindow);
  } else {
    if (!GTK_WIDGET_VISIBLE(g_dumpwindow)) {
      gtk_widget_show(g_dumpwindow);
    } else {
      gtk_widget_destroy(g_dumpwindow);
      g_dumpwindow = NULL;
    }
  }
}

void g_do()
{
  int i;
  struct g_harmonic g;

  GtkWidget *window;
  GtkWidget *box1;
  GtkWidget *box2;
  GtkWidget *button;
  GtkWidget *spin;
  GtkWidget *scale;
  GtkWidget *label;
  GtkWidget *separator;
  GtkObject *adjustment;

  g_initharmonic(&g, G_DEFHARM);

  /* main window */
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(g_destroy), NULL);
  gtk_window_set_title(GTK_WINDOW(window), "GTK Harmonics");
  gtk_container_border_width(GTK_CONTAINER(window), 0);

  /* main packing box */
  box1 = gtk_hbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(window), box1);
  gtk_widget_show(box1);

  /* left box for buttons */
  box2 = gtk_vbox_new(FALSE, 10);
  gtk_container_border_width(GTK_CONTAINER(box2), 10);
  gtk_box_pack_start(GTK_BOX(box1), box2, FALSE, TRUE, 0);
  gtk_widget_show(box2);

  /* label */
  label = gtk_label_new("frequency");
  gtk_box_pack_start(GTK_BOX(box2), label, FALSE, FALSE, 0);
  gtk_widget_show(label);

  /* spin stuff */
  adjustment = gtk_adjustment_new(G_DEFREQ, G_MINFREQ, G_STEP + G_MAXFREQ, G_STEP, G_PAGE, G_STEP);
  g.g_adjs[0] = adjustment;
  gtk_signal_connect(GTK_OBJECT(adjustment), "value_changed", GTK_SIGNAL_FUNC(g_notice), (gpointer) (&g));

  spin = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment), 20.0, 0);

  for (i = 1; i < g.g_harmonics; i++) {
    adjustment = gtk_adjustment_new(G_RANGE, 0.0, G_STEP + G_RANGE, G_STEP, G_PAGE, G_STEP);
    g.g_adjs[i] = adjustment;
#ifdef TRACE
    fprintf(stderr, "g_do(): The adjs[%d]=<%p>\n", 0, adjustment);
#endif
    gtk_signal_connect(GTK_OBJECT(adjustment), "value_changed", GTK_SIGNAL_FUNC(g_notice), (gpointer) (&g));
  }

  gtk_box_pack_start(GTK_BOX(box2), spin, FALSE, FALSE, 0);
  GTK_WIDGET_SET_FLAGS(spin, GTK_CAN_DEFAULT);
  gtk_widget_show(spin);

  /* mute */
  button = gtk_button_new_with_label("silence");
  gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(g_silence), NULL);
  gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
  gtk_widget_grab_default(button);
  gtk_widget_show(button);

  /* dump wave to file */
  button = gtk_button_new_with_label("dump");
  gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(g_dump), NULL);
  gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
  gtk_widget_show(button);

  /* exit program */
  button = gtk_button_new_with_label("quit");
  gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(g_destroy), NULL);
  gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
  gtk_widget_show(button);

  separator = gtk_vseparator_new();
  gtk_box_pack_start(GTK_BOX(box1), separator, FALSE, TRUE, 0);
  gtk_widget_show(separator);

  /* right box containing sliders */
  box2 = gtk_hbox_new(FALSE, 10);
  gtk_container_border_width(GTK_CONTAINER(box2), 10);
  gtk_box_pack_start(GTK_BOX(box1), box2, TRUE, TRUE, 0);
  gtk_widget_show(box2);

#ifdef TRACE
  fprintf(stderr, "g_do(): <%d> harmonics\n", g.g_harmonics);
#endif

  /* set up the sliders */
  for (i = 1; i < g.g_harmonics; i++) {

    scale = gtk_vscale_new(GTK_ADJUSTMENT(g.g_adjs[i]));
    gtk_range_set_update_policy(GTK_RANGE(scale), GTK_UPDATE_DISCONTINUOUS);
    gtk_scale_set_digits(GTK_SCALE(scale), 0);
    gtk_scale_set_draw_value(GTK_SCALE(scale), FALSE);
    gtk_widget_set_usize(GTK_WIDGET(scale), 40, 200);
    gtk_scale_set_value_pos(GTK_SCALE(scale), GTK_POS_BOTTOM);

    gtk_box_pack_start(GTK_BOX(box2), scale, TRUE, TRUE, 0);
    gtk_widget_show(scale);
  }

  gtk_widget_show(window);

  gtk_main();
}

/* 21J.gf83Iap.Q */
